@@ -3,26 +3,27 @@
 The code in this module is heavily inspired by django-nose:
 https://github.com/jbalogh/django-nose/
 """
+import os.path
 import sys
 import types
 
 
-def is_in_memory_db(connection):
-    """Return whether it makes any sense to use REUSE_DB with the backend of a
-    connection."""
-    # This is a SQLite in-memory DB. Those are created implicitly when
-    # you try to connect to them, so our test below doesn't work.
-    return connection.settings_dict['NAME'] == ':memory:'
-
-
 def test_database_exists_from_previous_run(connection):
-    # Check for sqlite memory databases
-    if is_in_memory_db(connection):
-        return False
-
     # Try to open a cursor to the test database
+    test_db_name = connection.creation._get_test_db_name()
+
+    # When using a real SQLite backend (via TEST_NAME), check if the file
+    # exists, because it gets created automatically.
+    if connection.settings_dict['ENGINE'] == 'django.db.backends.sqlite3':
+        if not os.path.exists(test_db_name):
+            return False
+
     orig_db_name = connection.settings_dict['NAME']
-    connection.settings_dict['NAME'] = connection.creation._get_test_db_name()
+    connection.settings_dict['NAME'] = test_db_name
+
+    # With SQLite memory databases the db never exists.
+    if connection.settings_dict['NAME'] == ':memory:':
+        return False
 
     try:
         connection.cursor()
@@ -46,13 +47,24 @@ def _monkeypatch(obj, method_name, new_method):
 
 
 def _get_db_name(db_settings, suffix):
-    if db_settings['ENGINE'] == 'django.db.backends.sqlite3':
-        return ':memory:'
+    "This provides the default test db name that Django uses."
+    from django import VERSION as DJANGO_VERSION
+
+    name = None
+    try:
+        if DJANGO_VERSION > (1, 7):
+            name = db_settings['TEST']['NAME']
+        elif DJANGO_VERSION < (1, 7):
+            name = db_settings['TEST_NAME']
+    except KeyError:
+        pass
+
+    if not name:
+        if db_settings['ENGINE'] == 'django.db.backends.sqlite3':
+            return ':memory:'
+        else:
+            name = 'test_' + db_settings['NAME']
 
-    if db_settings.get('TEST_NAME'):
-        name = db_settings['TEST_NAME']
-    else:
-        name = 'test_' + db_settings['NAME']
     if suffix:
         name = '%s_%s' % (name, suffix)
     return name
@@ -64,20 +76,21 @@ def monkey_patch_creation_for_db_suffix(suffix=None):
     if suffix is not None:
         def _get_test_db_name(self):
             """
-            Internal implementation - returns the name of the test DB that will be
-            created. Only useful when called from create_test_db() and
-            _create_test_db() and when no external munging is done with the 'NAME'
-            or 'TEST_NAME' settings.
+            Internal implementation - returns the name of the test DB that will
+            be created. Only useful when called from create_test_db() and
+            _create_test_db() and when no external munging is done with the
+            'NAME' or 'TEST_NAME' settings.
             """
             db_name = _get_db_name(self.connection.settings_dict, suffix)
             return db_name
 
         for connection in connections.all():
-
-            _monkeypatch(connection.creation, '_get_test_db_name', _get_test_db_name)
+            _monkeypatch(connection.creation,
+                         '_get_test_db_name', _get_test_db_name)
 
 
-def create_test_db_with_reuse(self, verbosity=1, autoclobber=False):
+def create_test_db_with_reuse(self, verbosity=1, autoclobber=False,
+                              keepdb=False, serialize=False):
     """
     This method is a monkey patched version of create_test_db that
     will not actually create a new database, but just reuse the
@@ -106,4 +119,5 @@ def monkey_patch_creation_for_db_reuse():
 
     for connection in connections.all():
         if test_database_exists_from_previous_run(connection):
-            _monkeypatch(connection.creation, 'create_test_db', create_test_db_with_reuse)
+            _monkeypatch(connection.creation, 'create_test_db',
+                         create_test_db_with_reuse)
