@@ -1,6 +1,9 @@
 import sys
+
 import pytest
 
+from .lazy_django import get_django_version
+
 
 def supported():
     import django.test.testcases
@@ -33,16 +36,24 @@ class LiveServer(object):
                 conn.allow_thread_sharing = True
                 connections_override[conn.alias] = conn
 
-        try:
-            from django.test.testcases import _StaticFilesHandler
-            static_handler_kwargs = {'static_handler': _StaticFilesHandler}
-        except ImportError:
-            static_handler_kwargs = {}
+        liveserver_kwargs = {'connections_override': connections_override}
+        from django.conf import settings
+        if ('django.contrib.staticfiles' in settings.INSTALLED_APPS and
+                get_django_version() >= (1, 7)):
+            from django.contrib.staticfiles.handlers import (
+                StaticFilesHandler)
+            liveserver_kwargs['static_handler'] = StaticFilesHandler
+        else:
+            try:
+                from django.test.testcases import _StaticFilesHandler
+            except ImportError:
+                pass
+            else:
+                liveserver_kwargs['static_handler'] = _StaticFilesHandler
 
         host, possible_ports = parse_addr(addr)
         self.thread = LiveServerThread(host, possible_ports,
-                                       connections_override=connections_override,
-                                       **static_handler_kwargs)
+                                       **liveserver_kwargs)
         self.thread.daemon = True
         self.thread.start()
         self.thread.is_ready.wait()
@@ -66,7 +77,7 @@ class LiveServer(object):
             return self.url
 
         def __add__(self, other):
-            return unicode(self) + other
+            return unicode(self) + other  # noqa: pyflakes on python3
     else:
         def __str__(self):
             return self.url
@@ -92,7 +103,7 @@ def parse_addr(specified_address):
         for port_range in port_ranges.split(','):
             # A port range can be of either form: '8000' or '8000-8010'.
             extremes = list(map(int, port_range.split('-')))
-            assert len(extremes) in [1, 2]
+            assert len(extremes) in (1, 2)
             if len(extremes) == 1:
                 # Port range of the form '8000'
                 possible_ports.append(extremes[0])
@@ -104,4 +115,4 @@ def parse_addr(specified_address):
         raise Exception(
             'Invalid address ("%s") for live server.' % specified_address)
 
-    return (host, possible_ports)
+    return host, possible_ports
