@@ -1,32 +1,33 @@
 """A py.test plugin which helps testing Django applications
 
 This plugin handles creating and destroying the test environment and
-test database and provides some useful text fixtues.
+test database and provides some useful text fixtures.
 """
 
 import os
-import sys
+
+import contextlib
 
 import pytest
 
 from .django_compat import is_django_unittest
-from .fixtures import (_django_db_setup, db, transactional_db, client,
-                       admin_client, rf, settings, live_server,
-                       _live_server_helper)
+from .fixtures import (_django_db_setup, _live_server_helper, admin_client,
+                       admin_user, client, db, django_user_model,
+                       django_username_field, live_server, rf, settings,
+                       transactional_db)
+from .lazy_django import django_settings_is_configured, skip_if_no_django
 
-from .lazy_django import skip_if_no_django, django_settings_is_configured
-
-
-(_django_db_setup, db, transactional_db, client, admin_client, rf,
- settings, live_server, _live_server_helper)
+# Silence linters for imported fixtures.
+(_django_db_setup, _live_server_helper, admin_client, admin_user, client, db,
+ django_user_model, django_username_field, live_server, rf, settings,
+ transactional_db)
 
 
 SETTINGS_MODULE_ENV = 'DJANGO_SETTINGS_MODULE'
 CONFIGURATION_ENV = 'DJANGO_CONFIGURATION'
 
 
-################ pytest hooks ################
-
+# ############### pytest hooks ################
 
 def pytest_addoption(parser):
     group = parser.getgroup('django')
@@ -52,15 +53,129 @@ def pytest_addoption(parser):
     parser.addini(SETTINGS_MODULE_ENV,
                   'Django settings module to use by pytest-django.')
 
-def _load_settings(config, options):
+    parser.addini('django_find_project',
+                  'Automatically find and add a Django project to the '
+                  'Python path.',
+                  default=True)
+
+import py
+import sys
+
+
+def _exists(path, ignore=EnvironmentError):
+    try:
+        return path.check()
+    except ignore:
+        return False
+
+
+PROJECT_FOUND = ('pytest-django found a Django project in %s '
+                 '(it contains manage.py) and added it to the Python path.\n'
+                 'If this is wrong, add "django_find_project = false" to '
+                 'pytest.ini and expliclity manage your Python path.')
+
+PROJECT_NOT_FOUND = ('pytest-django could not find a Django project '
+                     '(no manage.py file could be found). You must '
+                     'expliclity add your Django project to the Python path '
+                     'to have it picked up.')
+
+PROJECT_SCAN_DISABLED = ('pytest-django did not search for Django '
+                         'projects since it is disabled in the configuration '
+                         '("django_find_project = false")')
+
+
+@contextlib.contextmanager
+def _handle_import_error(extra_message):
+    try:
+        yield
+    except ImportError as e:
+        django_msg = (e.args[0] + '\n\n') if e.args else ''
+        msg = django_msg + extra_message
+        raise ImportError(msg)
+
+
+def _add_django_project_to_path(args):
+    args = [x for x in args if not str(x).startswith("-")]
+
+    if not args:
+        args = [py.path.local()]
+
+    for arg in args:
+        arg = py.path.local(arg)
+
+        for base in arg.parts(reverse=True):
+            manage_py_try = base.join('manage.py')
+
+            if _exists(manage_py_try):
+                sys.path.insert(0, str(base))
+                return PROJECT_FOUND % base
+
+    return PROJECT_NOT_FOUND
+
+
+def _setup_django():
+    import django
+
+    if hasattr(django, 'setup'):
+        django.setup()
+    else:
+        # Emulate Django 1.7 django.setup() with get_models
+        from django.db.models import get_models
+
+        get_models()
+
+
+def _parse_django_find_project_ini(x):
+    if x in (True, False):
+        return x
+
+    x = x.lower()
+    possible_values = {'true': True,
+                       'false': False,
+                       '1': True,
+                       '0': False}
+
+    try:
+        return possible_values[x]
+    except KeyError:
+        raise ValueError('%s is not a valid value for django_find_project. '
+                         'It must be one of %s.'
+                         % (x, ', '.join(possible_values.keys())))
+
+
+def pytest_load_initial_conftests(early_config, parser, args):
+    # Register the marks
+    early_config.addinivalue_line(
+        'markers',
+        'django_db(transaction=False): Mark the test as using '
+        'the django test database.  The *transaction* argument marks will '
+        "allow you to use real transactions in the test like Django's "
+        'TransactionTestCase.')
+    early_config.addinivalue_line(
+        'markers',
+        'urls(modstr): Use a different URLconf for this test, similar to '
+        'the `urls` attribute of Django `TestCase` objects.  *modstr* is '
+        'a string specifying the module of a URL config, e.g. '
+        '"my_app.test_urls".')
+
+    options = parser.parse_known_args(args)
+
+    django_find_project = _parse_django_find_project_ini(
+        early_config.getini('django_find_project'))
+
+    if django_find_project:
+        _django_project_scan_outcome = _add_django_project_to_path(args)
+    else:
+        _django_project_scan_outcome = PROJECT_SCAN_DISABLED
+
     # Configure DJANGO_SETTINGS_MODULE
     ds = (options.ds or
-          config.getini(SETTINGS_MODULE_ENV) or
+          early_config.getini(SETTINGS_MODULE_ENV) or
           os.environ.get(SETTINGS_MODULE_ENV))
 
     # Configure DJANGO_CONFIGURATION
     dc = (options.dc or
-          config.getini(CONFIGURATION_ENV) or
+          early_config.getini(CONFIGURATION_ENV) or
           os.environ.get(CONFIGURATION_ENV))
 
     if ds:
@@ -73,47 +188,26 @@ def _load_settings(config, options):
             import configurations.importer
             configurations.importer.install()
 
+        # Forcefully load django settings, throws ImportError or
+        # ImproperlyConfigured if settings cannot be loaded.
         from django.conf import settings
-        try:
+
+        with _handle_import_error(_django_project_scan_outcome):
             settings.DATABASES
-        except ImportError:
-            e = sys.exc_info()[1]
-            raise pytest.UsageError(*e.args)
+
+        _setup_django()
 
 
-if pytest.__version__[:3] >= "2.4":
-    def pytest_load_initial_conftests(early_config, parser, args):
-        _load_settings(early_config, parser.parse_known_args(args))
-
-
-def pytest_configure(config):
-    # Register the marks
-    config.addinivalue_line(
-        'markers',
-        'django_db(transaction=False): Mark the test as using '
-        'the django test database.  The *transaction* argument marks will '
-        "allow you to use real transactions in the test like Django's "
-        'TransactionTestCase.')
-    config.addinivalue_line(
-        'markers',
-        'urls(modstr): Use a different URLconf for this test, similar to '
-        'the `urls` attribute of Django `TestCase` objects.  *modstr* is '
-        'a string specifying the module of a URL config, e.g. '
-        '"my_app.test_urls".')
-
-    if pytest.__version__[:3] < "2.4":
-        _load_settings(config, config.option)
-
-
-################ Autouse fixtures ################
+@pytest.mark.trylast
+def pytest_configure():
+    if django_settings_is_configured():
+        _setup_django()
 
 
 @pytest.fixture(autouse=True, scope='session')
-def _django_runner(request):
-    """Create the django runner, internal to pytest-django
-
-    This does important things like setting up the local memory email
-    backend etc.
+def _django_test_environment(request):
+    """
+    Ensure that Django is loaded and has its testing environment setup
 
     XXX It is a little dodgy that this is an autouse fixture.  Perhaps
         an email fixture should be requested in order to be able to
@@ -123,17 +217,11 @@ def _django_runner(request):
         we need to follow this model.
     """
     if django_settings_is_configured():
-        import django
-        # Django >= 1.7: Call django.setup() to initialize Django
-        setup = getattr(django, 'setup', lambda: None)
-        setup()
-
-        from django.test.simple import DjangoTestSuiteRunner
-
-        runner = DjangoTestSuiteRunner(interactive=False)
-        runner.setup_test_environment()
-        request.addfinalizer(runner.teardown_test_environment)
-        return runner
+        from django.conf import settings
+        from .compat import setup_test_environment, teardown_test_environment
+        settings.DEBUG = False
+        setup_test_environment()
+        request.addfinalizer(teardown_test_environment)
 
 
 @pytest.fixture(autouse=True, scope='session')
@@ -181,7 +269,7 @@ def _django_db_marker(request):
 def _django_setup_unittest(request, _django_cursor_wrapper):
     """Setup a django unittest, internal to pytest-django"""
     if django_settings_is_configured() and is_django_unittest(request.node):
-        request.getfuncargvalue('_django_runner')
+        request.getfuncargvalue('_django_test_environment')
         request.getfuncargvalue('_django_db_setup')
         _django_cursor_wrapper.enable()
         request.addfinalizer(_django_cursor_wrapper.disable)
@@ -215,7 +303,7 @@ def _django_set_urlconf(request):
         request.addfinalizer(restore)
 
 
-################ Helper Functions ################
+# ############### Helper Functions ################
 
 
 class CursorManager(object):
